@@ -15,7 +15,7 @@ def test():
 
 class TestReadWrite(unittest.TestCase):
     def setUp(self):
-        
+
 
 
         nvec = 2
@@ -24,6 +24,7 @@ class TestReadWrite(unittest.TestCase):
         # all currently available types, scalar, 1-d and 2-d array columns
         dtype=[('u1scalar','u1'),
                ('i1scalar','i1'),
+               ('b1scalar','b1'),
                ('u2scalar','u2'),
                ('i2scalar','i2'),
                ('u4scalar','u4'),
@@ -31,9 +32,12 @@ class TestReadWrite(unittest.TestCase):
                ('i8scalar','i8'),
                ('f4scalar','f4'),
                ('f8scalar','>f8'),
+               ('c8scalar','c8'), # complex, two 32-bit
+               ('c16scalar','c16'), # complex, two 32-bit
 
                ('u1vec','u1',nvec),
                ('i1vec','i1',nvec),
+               ('b1vec','b1',nvec),
                ('u2vec','u2',nvec),
                ('i2vec','i2',nvec),
                ('u4vec','u4',nvec),
@@ -41,9 +45,12 @@ class TestReadWrite(unittest.TestCase):
                ('i8vec','i8',nvec),
                ('f4vec','f4',nvec),
                ('f8vec','f8',nvec),
+               ('c8vec','c8',nvec),
+               ('c16vec','c16',nvec),
  
                ('u1arr','u1',ashape),
                ('i1arr','i1',ashape),
+               ('b1arr','b1',ashape),
                ('u2arr','u2',ashape),
                ('i2arr','i2',ashape),
                ('u4arr','u4',ashape),
@@ -51,6 +58,8 @@ class TestReadWrite(unittest.TestCase):
                ('i8arr','i8',ashape),
                ('f4arr','f4',ashape),
                ('f8arr','f8',ashape),
+               ('c8arr','c8',ashape),
+               ('c16arr','c16',ashape),
 
                ('Sscalar',Sdtype),
                ('Svec',   Sdtype, nvec),
@@ -63,19 +72,39 @@ class TestReadWrite(unittest.TestCase):
         nrows=4
         data=numpy.zeros(nrows, dtype=dtype)
 
-        for t in ['u1','i1','u2','i2','u4','i4','i8','f4','f8']:
-            data[t+'scalar'] = 1 + numpy.arange(nrows, dtype=t)
-            data[t+'vec'] = 1 + numpy.arange(nrows*nvec,dtype=t).reshape(nrows,nvec)
-            arr = 1 + numpy.arange(nrows*ashape[0]*ashape[1],dtype=t)
+        dtypes=['u1','i1','u2','i2','u4','i4','i8','f4','f8','c8','c16']
+        for t in dtypes:
+            if t in ['c8','c16']:
+                data[t+'scalar'] = [complex(i+1,(i+1)*2) for i in xrange(nrows)]
+                vname=t+'vec'
+                for row in xrange(nrows):
+                    for i in xrange(nvec):
+                        index=(row+1)*(i+1)
+                        data[vname][row,i] = complex(index,index*2)
+                aname=t+'arr'
+                for row in xrange(nrows):
+                    for i in xrange(ashape[0]):
+                        for j in xrange(ashape[1]):
+                            index=(row+1)*(i+1)*(j+1)
+                            data[aname][row,i,j] = complex(index,index*2)
+
+            else:
+                data[t+'scalar'] = 1 + numpy.arange(nrows, dtype=t)
+                data[t+'vec'] = 1 + numpy.arange(nrows*nvec,dtype=t).reshape(nrows,nvec)
+                arr = 1 + numpy.arange(nrows*ashape[0]*ashape[1],dtype=t)
+                data[t+'arr'] = arr.reshape(nrows,ashape[0],ashape[1])
+
+        for t in ['b1']:
+            data[t+'scalar'] = (numpy.arange(nrows) % 2 == 0).astype(t)
+            data[t+'vec'] = (numpy.arange(nrows*nvec) % 2 == 0).astype(t).reshape(nrows,nvec)
+            arr = (numpy.arange(nrows*ashape[0]*ashape[1]) % 2 == 0).astype(t)
             data[t+'arr'] = arr.reshape(nrows,ashape[0],ashape[1])
 
 
-
         # strings get padded when written to the fits file.  And the way I do
-        # the read, I real all bytes (ala mrdfits) so the spaces are preserved.
+        # the read, I read all bytes (ala mrdfits) so the spaces are preserved.
         # 
-        # so for comparisons, we need to pad out the strings with blanks so we
-        # can compare
+        # so we need to pad out the strings with blanks so we can compare
 
         data['Sscalar'] = ['%-6s' % s for s in ['hello','world','good','bye']]
         data['Svec'][:,0] = '%-6s' % 'hello'
@@ -229,10 +258,11 @@ class TestReadWrite(unittest.TestCase):
         """
 
         fname=tempfile.mktemp(prefix='fitsio-ImageWrite-',suffix='.fits')
+        dtypes=['u1','i1','u2','i2','<u4','i4','i8','>f4','f8']
         try:
             with fitsio.FITS(fname,'rw',clobber=True) as fits:
                 # note mixing up byte orders a bit
-                for dtype in ['u1','i1','u2','i2','<u4','i4','i8','>f4','f8']:
+                for dtype in dtypes:
                     data = numpy.arange(5*20,dtype=dtype).reshape(5,20)
                     header={'DTYPE':dtype,'NBYTES':data.dtype.itemsize}
                     fits.write_image(data, header=header)
@@ -247,6 +277,11 @@ class TestReadWrite(unittest.TestCase):
                             v = v.strip()
                             rv = rv.strip()
                         self.assertEqual(v,rv,"testing equal key '%s'" % k)
+
+            with fitsio.FITS(fname) as fits:
+                for i in xrange(len(dtypes)):
+                    self.assertEqual(fits[i].is_compressed(), False, "not compressed")
+
         finally:
             if os.path.exists(fname):
                 os.remove(fname)
@@ -282,20 +317,39 @@ class TestReadWrite(unittest.TestCase):
         Test a basic image write, data and a header, then reading back in to
         check the values
         """
+        nrows=30
+        ncols=100
+        tile_dims=[5,10]
         compress='rice'
         fname=tempfile.mktemp(prefix='fitsio-ImageWrite-',suffix='.fits.fz')
+        dtypes = ['u1','i1','u2','i2','u4','i4','f4','f8']
+
         try:
             with fitsio.FITS(fname,'rw',clobber=True) as fits:
                 # note i8 not supported for compressed!
-                dtypes = ['u1','i1','u2','i2','u4','i4','f4','f8']
 
                 for dtype in dtypes:
-                    data = numpy.arange(5*20,dtype=dtype).reshape(5,20)
+                    data = numpy.arange(nrows*ncols,dtype=dtype).reshape(nrows,ncols)
                     fits.write_image(data, compress=compress)
                     #fits.reopen()
                     rdata = fits[-1].read()
 
-                    self.compare_array(data, rdata, "%s compressed images ('%s')" % (compress,dtype))
+                    self.compare_array(data, rdata,
+                                       "%s compressed images ('%s')" % (compress,dtype))
+
+
+                    fits.write_image(data, compress=compress, tile_dims=tile_dims)
+                    #fits.reopen()
+                    rdata = fits[-1].read()
+
+                    self.compare_array(data, rdata,
+                                       ("%s tile dims compressed images "
+                                        "('%s')" % (compress,dtype)))
+
+            with fitsio.FITS(fname) as fits:
+                for ii in xrange(len(dtypes)):
+                    i=ii+1
+                    self.assertEqual(fits[i].is_compressed(), True, "is compressed")
 
         finally:
             if os.path.exists(fname):
@@ -843,7 +897,37 @@ class TestReadWrite(unittest.TestCase):
                 #pass
                 os.remove(fname)
 
+    def testTableIter(self):
+        """
+        Test iterating over rows of a table
+        """
 
+        fname=tempfile.mktemp(prefix='fitsio-TableIter-',suffix='.fits')
+        try:
+            with fitsio.FITS(fname,'rw',clobber=True) as fits:
+
+                try:
+                    fits.write_table(self.data, header=self.keys, extname='mytable')
+                    write_success=True
+                except:
+                    write_success=False
+
+                self.assertTrue(write_success,"testing write does not raise an error")
+                if not write_success:
+                    skipTest("cannot test result if write failed")
+
+            # one row at a time
+            with fitsio.FITS(fname) as fits:
+                hdu = fits["mytable"]
+                i=0
+                for row_data in hdu:
+                    self.compare_rec(self.data[i], row_data, "table data")
+                    i+=1
+
+        finally:
+            if os.path.exists(fname):
+                #pass
+                os.remove(fname)
 
     def testAsciiTableWriteRead(self):
         """
@@ -1378,3 +1462,6 @@ class TestReadWrite(unittest.TestCase):
 
 
 
+if __name__ == '__main__':
+    test()
+
